@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 ##############################################################
 # smscon_master - remote control master for Nokia N900 phone #
-# $Id: smscon_master.py 91 2011-12-26 10:36:05Z yablacky $
+# $Id: smscon_master.py 112 2011-12-29 01:37:20Z yablacky $
 ##############################################################
 
 import os
@@ -142,7 +142,7 @@ def ShowHiddenOptions():
     print '   -init!             : Create default %s files. Do not upgrade existing ones.' % ' & '.join((ConfigDisplayName, ScriptDisplayName))
     print '   -get name "default": Get user setting'
     print '   -test "test"       : Run developer test (gps1, gps2, ssh, ssh-off, cam, sms, email1, email2, call, script)'
-    print '   -comtest "command" : Run %s command as if received as SMS message' % NAME
+    print '   -comtest "command" : Run %s command as if received as SMS message (no prefix/suffix required)' % NAME
     print '   -devconfig password: Show uncrypted %s file. MASTERNUMBER is password' % ConfigDisplayName
 
 ##############################################################
@@ -262,17 +262,14 @@ def DoCommandLine():
         elif Mode == '-boot':       ok = CreateBootfile()
         elif Mode == '-unboot':     ok = DeleteFile(BootPath, BootFile)
 
-        elif Mode in('-test', '-comtest'): # /* ONLY FOR DEVELOPMENT USAGE */
+        elif Mode in('-test', '-comtest'):
             if len(args) == 0: return TellError("%s requires argument." % Mode)
             Func = args.pop(0)
-            if daemon.stop():
-                time.sleep(DelaySeconds)
-            LOGGER.info('"%s" TEST BEGINS --------------' % Func.upper())
-            daemon.start(Mode = Mode, Function = Func)
-            time.sleep(DelaySeconds)
             if not daemon.runs():
-                return TellError ('"%s" TEST: Daemon %s failed to start.' % (Func.upper(), DaemonFile))
-            LOGGER.info( '"%s" TEST: Daemon %s now active.' % (Func.upper(), DaemonFile) )
+                daemon.start(Mode, Func)
+            elif not daemon.writeCliCommand(Mode, Func):
+                return TellError('%s %s failed to start.' % (Mode[1:], Func))
+            LOGGER.info('%s %s started...' % (Mode[1:], Func))
 
         elif Mode == '-del':
             if len(args) == 0: return TellError("%s requires argument." % Mode)
