@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 ##############################################################
 # smscon_service - remote control daemon for Nokia N900 phone#
-# $Id: smscon_service.py 87 2011-12-25 14:54:03Z yablacky $
+# $Id: smscon_service.py 112 2011-12-29 01:37:20Z yablacky $
 ##############################################################
 
 # standard modules
@@ -28,7 +28,7 @@ from email.MIMEImage import MIMEImage
 from email.MIMEMultipart import MIMEMultipart
 
 import smsconlib as smscon
-from smsconlib import InstPath, LogPath, PhotoName, MapName, AudioFile, DaemonFile 
+from smsconlib import InstPath, LogPath, PhotoName, MapName, AudioFile, DaemonFile, smscon_daemon
 from smsconlib import IF, IsHostReachable, IsInternetReachable, IsSamePhoneNumber, IsFileReadable, DeleteFile, shellArg, pidof, SendSMSviaGSMmodem
 
 LOGGER = smscon.LOGGER = smscon.StartLogging('DAEMON', withConsole = False)
@@ -43,9 +43,8 @@ LastKnownGoodConfigFile     = '.%s_lkg' % smscon.NAME
 #    variables
 
 GpsCoordinateList           = []
-GpsIsActive                 = False                             # true if GPS device is enabled
+GpsMode                     = False                             # False if not active or mode-name if GPS device is enabled
 GpsIsTest                   = False                             # true if GPS device is enabled from test mode command
-GpsDataOK                   = False                             # true if GPS device receives valid coordinates
 GpsStartTime                = 0
 
 ConnectionType              = None 
@@ -53,7 +52,6 @@ ConnectionIAP               = None
 ConnectionName              = None
 ConnectionBehaviour         = None
 
-IsTestModeActive            = False
 IsKeyboardSliderCheckActive = False
 IsBatteryStatusCheckActive  = False
 IsCheckHostActive           = False
@@ -64,6 +62,7 @@ CheckHostLastCommand        = '-1'      # "restart"
 CheckHostStatusCount        = 0         # positive: successes, negative: errors
 CheckHostStatusTimestamp    = time.gmtime()
 
+daemon = None               # Our daemon object
 smscnf = None               # the UserConfiguration actually used by the daemon.
 
 #############################################
@@ -572,6 +571,7 @@ def ProcessCommand(Command, SenderPhoneNumber=None, CommandOrigin=None, ExtraInf
     elif Command == smscnf.COM_REMOTEON:
         if smscnf.REMOTEUSER == '' and smscnf.REMOTEPASSWORD == '':
             LOGGER.warning('No user/password entered for enabling SSH connection.')
+            RevealThat('ssh', 'No user/password entered for enabling SSH connection.')
         else:
             if not IsInternetReachable():
                 SetNetworkConnectionType('GPRS') # enable GPRS data network
@@ -581,6 +581,7 @@ def ProcessCommand(Command, SenderPhoneNumber=None, CommandOrigin=None, ExtraInf
     elif Command == smscnf.COM_REMOTEOFF:
         if smscnf.REMOTEUSER == '' and smscnf.REMOTEPASSWORD == '':
             LOGGER.warning('No user/password entered for disabling SSH connection.')
+            RevealThat('ssh', 'No user/password entered for disabling SSH connection.')
         else:
             DisableSSHcontrol()
             
@@ -800,7 +801,7 @@ def PerformDirectShellCommand(Command):
     try:
         Result = os.popen(Command).read()
     except:
-        LOGGER.Error('Failed to run shell command (%s).' % Command)
+        LOGGER.error('Failed to run shell command (%s).' % Command)
         RevealThat('shell', ['command failed: %s.' % Command])        
     else:
         LOGGER.info('Shell command (%s) executed.' % Command)
@@ -1082,7 +1083,7 @@ def GetGPRSname():
     CurrentIMSI = smscon.GetPhoneCode('IMSI')
 
     for Network in NetworkList:
-        Type, IAP, Name, Imsi = Network[0:4]
+        Type, IAP, Name, Imsi = (Network + ['NONE', 'NONE'])[0:4]
         if Type == 'GPRS' and Imsi == CurrentIMSI:
             LOGGER.debug('original IAP=[%s]' % IAP)
             return Type, IAP.replace('@32@', ' '), Name
@@ -1113,7 +1114,7 @@ def GetNetworks():
     Get list of all available data connections (stored under "Internet connections" in menu).
     @return list of (Type, IAP, Name [, Imsi, NameAP]) tuples.
     """
-    Output = os.popen('gconftool -R /system/osso/connectivity/IAP | grep IAP').read()
+    Output = os.popen('gconftool -R /system/osso/connectivity/IAP | grep /IAP/').read()
     E = re.compile('/system/osso/connectivity/IAP/(\S+):')
     IAPlist = E.findall(Output) 
 
@@ -1121,6 +1122,8 @@ def GetNetworks():
     NetworkList = []
 
     for IAP in IAPlist:
+        if IAP == '0000-0000-0000-0000':
+            continue
         # get connection type
         Type = os.popen('gconftool -g /system/osso/connectivity/IAP/%s/type' % IAP).read().replace('_INFRA', '').strip('\n')
 
@@ -1150,7 +1153,7 @@ def GetNetworks():
             # add WLAN connection to list
             LOGGER.debug( 'Type=%s, IAP=%s, Name=%s' % (Type, IAP, Name) )
             NetworkList.append( (Type, IAP, Name) )
-        
+
     return NetworkList
 
 def RememberNetworkSettings():
@@ -1190,8 +1193,9 @@ def SetAutoConnectBehaviour(NewBehaviour):
     """
     Set Nokia phone data connection behavior.
     The old behaviour is saved in global ConnectionBehaviour and can be used to restore.
+    @param string NewBehaviour One of [ASK|WLAN|GPRS|ANY]
     @return bool True if behaviour changed.
-    """    
+    """
 
     def _getConnectBehaviour():
         """
@@ -1210,6 +1214,7 @@ def SetAutoConnectBehaviour(NewBehaviour):
     def _setConnectBehaviour(Behaviour):
         """
         Set new Nokia device data connection behaviour (ASK|WLAN|GPRS|ANY).
+        @param string Behaviour One of [ASK|WLAN|GPRS|ANY]
         @return void
         """
         DelaySeconds = 2
@@ -1235,7 +1240,7 @@ def SetAutoConnectBehaviour(NewBehaviour):
         ConnectionBehaviour = OldBehaviour
 
     if NewBehaviour == None:
-        LOGGER.info('Changing internet connection behaviour not enabled (is "%s").' % OldBehaviour)
+        LOGGER.info('Changing internet connection behaviour not enabled (still is "%s").' % OldBehaviour)
         return False
     elif NewBehaviour == OldBehaviour:
         LOGGER.info('Internet connection behaviour already set to "%s".' % NewBehaviour)
@@ -1349,15 +1354,12 @@ def EnableGPStracking(Mode):
     Start the GPS device mainloop.
     """
     global GpsMode
-    GpsMode = Mode
-    
-    global GpsIsActive
-    if not GpsIsActive:
-        GpsIsActive = True
+    if not GpsMode:
+        GpsMode = Mode
         # enable GPS device control
         gobject.idle_add(GPSstart, GpsControl)
     else:
-        LOGGER.warning('GPS device already active.')
+        LOGGER.warning('GPS device already active in %s mode.' % GpsMode)
 
 def DisableGPStracking():
     """
@@ -1365,9 +1367,9 @@ def DisableGPStracking():
     """
     GpsData = GpsControl
 
-    global GpsIsActive
-    if GpsIsActive:
-        GpsIsActive = False
+    global GpsMode
+    if GpsMode:
+        GpsMode = False
         # stop the GPS device
         GpsData.stop()
         LOGGER.info('Stopped acquiering GPS coordinates.')
@@ -1378,9 +1380,6 @@ def GPSstart(GpsData):
     """
     Start the GPS device.
     """
-    global GpsDataOK
-    GpsDataOK = False
-
     if GpsMode == 'single':
         LOGGER.info('Starting GPS device in location mode.')
     elif GpsMode == 'multi':
@@ -1396,14 +1395,13 @@ def OnGPSstop(GpsControl, GpsData):
     """
     Stop the GPS device.
     """
-    global GpsIsActive
-    GpsIsActive = False
+    global GpsMode
+    GpsMode = False
 
     global GpsIsTest
     if GpsIsTest:
         GpsIsTest = False
         RestoreNetworkSettings()
-        ExitDaemon()        
     
     LOGGER.info('Stopping GPS device.')
     GpsData.quit()
@@ -1413,39 +1411,43 @@ def OnGPSchanged(GpsDevice, GpsData):
     Get GPS device data.
     """
     global GpsCoordinateList
-    global GpsDataOK
     global GpsStartTime
 
-    if not GpsIsActive:
+    if not GpsMode:
         return
-    # check GPS device timout
+
     TimePassedSeconds = time.time() - GpsStartTime
+    LOGGER.debug('GSP GpsStartTime = %s.' % GpsStartTime)
+    LOGGER.debug('GpsDevice.status = %s' % GpsDevice.status) # LOCATION_GPS_DEVICE_STATUS_NO_FIX  or  LOCATION_GPS_DEVICE_STATUS_FIX
+
+    GpsDataOK = True
+    if GpsDataOK:
+        GpsDataOK = (GpsDevice.fix[1] & location.GPS_DEVICE_LATLONG_SET) != 0
+        if not GpsDataOK:
+            LOGGER.info('Waiting for GPS satellite fix (search time = %.1f sec.).' % TimePassedSeconds)
+
+    if GpsDataOK:
+        # GPS device has fix.
+        Latitude  = GpsDevice.fix[4]
+        Longitude = GpsDevice.fix[5]
+        Accuracy  = GpsDevice.fix[6] / 100
+    
+        # Check if coordinates acquired by GPS device are valid
+        GpsDataOK = str(Latitude) != 'nan' and str(Longitude) != 'nan' and str(Accuracy) != 'nan'
+        if not GpsDataOK:
+            LOGGER.info('Waiting for valid GPS coordinate (search time = %.1f sec.).' % TimePassedSeconds)
+
     if not GpsDataOK:
-        LOGGER.debug('GSP GpsStartTime = %s.' % GpsStartTime)
-        if TimePassedSeconds >= float(smscnf.GPSTIMEOUT):
+        # check GPS device timout
+        if TimePassedSeconds >= int(smscnf.GPSTIMEOUT):
             LOGGER.error(     'GPS device timeout after %.1f sec.' % TimePassedSeconds)
             RevealThat('GPS', 'GPS device timeout after %.1f sec.' % TimePassedSeconds, smscnf.GPSSEND)                    
             # stop the GPS device
             GpsData.stop()
-            return
+        return
 
-    # adjust start time of GPS device
+    # Now we have really data, adjust timeout timer
     GpsStartTime = time.time()
-    LOGGER.debug('GpsDevice.status = %s' % GpsDevice.status) # LOCATION_GPS_DEVICE_STATUS_NO_FIX  or  LOCATION_GPS_DEVICE_STATUS_FIX
-
-    if (GpsDevice.fix[1] & location.GPS_DEVICE_LATLONG_SET) == 0:
-        LOGGER.info('Waiting for GPS device satellite fix.')
-        return
-    # GPS device has fix.
-    Latitude  = GpsDevice.fix[4]
-    Longitude = GpsDevice.fix[5]
-    Accuracy  = GpsDevice.fix[6] / 100
-
-    # Check if coordinates acquired by GPS device are valid
-    GpsDataOK = str(Latitude) != 'nan' and str(Longitude) != 'nan' and str(Accuracy) != 'nan'
-    if not GpsDataOK:
-        LOGGER.info('Waiting for valid GPS coordinate (search time = %.1f sec.).' % TimePassedSeconds)
-        return
 
     if GpsMode == 'single':
         if len(GpsCoordinateList) == 0:
@@ -1486,8 +1488,8 @@ def OnGPSerror(GpsControl, GpsError, GpsData):
     """
     GPS device error detected.
     """
-    global GpsIsActive
-    GpsIsActive = False
+    global GpsMode
+    GpsMode = False
 
     if GpsError == location.ERROR_USER_REJECTED_DIALOG:
         LOGGER.error("GPS device error (User didn't enable requested methods).")
@@ -1621,34 +1623,37 @@ def ValidateIMSI():
 
 #############################################
 
-def DoCommandLine():
+def DoCommandLine(args):
     """
     Check arguments for smscon_daemon and perform corresponding actions.
-    @return bool True if ok
+    @return int Status where 0|False is fatal error, 1 is success, 2 is success with test mode
     """
-    global IsTestModeActive;
+
+    DoCommandLine.BAD = False
+    DoCommandLine.OK = 1
+    DoCommandLine.TESTMODE = 2
 
     def TellError(Msg):
         LOGGER.error(Msg)
         return False
 
-    def PrepareNetwork():
+    def PrepareTestModeNetwork():
         if  GetNetworkConnectionType() != 'WLAN':
             SetNetworkConnectionType('GPRS')
             SetAutoConnectBehaviour('GPRS') # disable switching to other network
 
-    args = sys.argv[1:]
+    CommandLineStatus = DoCommandLine.OK;
     if len(args) > 0:
         Mode = args.pop(0)
         if Mode == '-test':
             if len(args) == 0: return TellError('%s requires argument.' % Mode)
             Func = args.pop(0)
-            IsTestModeActive = True
-    
+            CommandLineStatus = CommandLineStatus and DoCommandLine.TESTMODE
+
             if Func == 'gps1' or Func == 'gps2': 
                 RememberNetworkSettings()
-                PrepareNetwork();
-    
+                PrepareTestModeNetwork();
+
                 global GpsIsTest
                 GpsIsTest = True
                 
@@ -1656,22 +1661,22 @@ def DoCommandLine():
                     EnableGPStracking('single')
                 elif Func == 'gps2': # test mode for gps (tracking mode)
                     EnableGPStracking('multi')
-                return True
-    
+                return CommandLineStatus
+
             if Func == 'ssh': # test mode for ssh connection
                 RememberNetworkSettings()
-                PrepareNetwork();
+                PrepareTestModeNetwork();
                 DisableSSHcontrol(Verbose = False)
                 EnableSSHcontrol()
-                return True
+                return CommandLineStatus
 
             if Func == 'ssh-off': # test for ssh disconnect
                 DisableSSHcontrol(Verbose = True)
-                return True
+                return CommandLineStatus
 
             if Func == 'cam': # test mode for camera
                 RememberNetworkSettings()
-                PrepareNetwork();
+                PrepareTestModeNetwork();
                 PerformCamera()
                 RestoreNetworkSettings()
                 SetAutoConnectBehaviour(ConnectionBehaviour)
@@ -1684,7 +1689,7 @@ def DoCommandLine():
 
             elif Func in ('email1', 'email2'): 
                 RememberNetworkSettings()
-                PrepareNetwork();
+                PrepareTestModeNetwork();
 
                 if Func == 'email1': # test mode1 for email (text only)
                     EMAILsend(EmailTo = smscnf.EMAILADDRESS,
@@ -1713,21 +1718,20 @@ def DoCommandLine():
             else:
                 LOGGER.error('Test option error (%s).' % Func)
 
-        # /* ONLY FOR DEVELOPMENT USAGE */
         elif Mode == '-comtest':
             if len(args) == 0: return TellError('%s requires argument.' % Mode)
             Command = args.pop(0)
-            IsTestModeActive = True
+            # Note: does not enter test mode!
 
             LOGGER.warning('Simulating received SMS message from MASTERNUMBER with "%s" command.' % Command)
             ProcessCommand(Command, SenderPhoneNumber = smscnf.MASTERNUMBER,
                            ExtraInfo = 'SIMULATION!', LogIfCmdUnknown = True)    
         else:
             LOGGER.warning('Option error (%s) - ignored' % Mode)
-            return True
+            return CommandLineStatus
     if len(args) > 0:
         LOGGER.warning('Options ignored: %s' % ' '.join(args))
-    return True                
+    return CommandLineStatus
 
 #############################################
 
@@ -1737,9 +1741,9 @@ def CheckBootLoader(WithNotification = True):
     @return void.
     """
     if IsFileReadable(smscon.BootPath, smscon.BootFile):
-        LOGGER.info('%s auto-loads at boot. OK.' % smscon.NAME)
+        LOGGER.info('%s auto-loads at boot. OK. %s' % (smscon.NAME, IF(WithNotification, '', 'Silent check.')))
     else:
-        LOGGER.warning(    "%s does not auto-load at boot." % smscon.NAME)
+        LOGGER.warning("%s does not auto-load at boot. %s" % (smscon.NAME, IF(WithNotification, '', 'Silent check.')))
         if WithNotification:
             RevealThat('boot', "%s does not auto-load at boot." % smscon.NAME)        
 
@@ -1899,6 +1903,9 @@ def InstallSignalReceivers():
     global IsKeyboardSliderCheckActive
     global IsBatteryStatusCheckActive
     global IsCheckHostActive
+    global daemon
+
+    daemon.openCommandChannel('r')
 
     if (not IsKeyboardSliderCheckActive) == (smscnf.KEYBOARDDETECT == 'yes'):
             IsKeyboardSliderCheckActive   =  smscnf.KEYBOARDDETECT == 'yes'
@@ -1971,11 +1978,33 @@ def LoadConfiguration():
         InstallSignalReceivers()
     return True
 
+def OnLocalCommandAvailable():
+    """
+    Try to read commands from local command pipe and execute them.
+    @return void  
+    """
+    global daemon
+
+    while True:
+        Command = daemon.readCommand()
+        if Command == '':
+            break
+        cliArgs = daemon.isCliCommand(Command)
+        if cliArgs:
+            LOGGER.info("----- TEST '%s' BEGINS -----" % ' '.join(cliArgs))
+            DoCommandLine(cliArgs[:])
+            LOGGER.info("----- TEST '%s' DONE -----" % ' '.join(cliArgs))
+        else:
+            LOGGER.warning('Unknown command "%s" found in local command channel. Ignored.' % Command)
+
 def main():
     """
     Main function of smscon daemon. Does not return to caller.
     """
     global smscnf, bus, dev_obj, GpsControl
+    global daemon
+
+    daemon = smscon_daemon()
 
     LOGGER.critical('%s is starting.' % DaemonFile)
     
@@ -1983,6 +2012,17 @@ def main():
                     lambda signum, stack_frame: ExitDaemon("on signal %s" % str(signum))  )
     signal.signal(smscon.smscon_daemon.TELL_RELOAD,
                     lambda signum, stack_frame: LoadConfiguration()                       )
+
+    def commandSignalHandler(signum, stack_frame):
+        """
+        Signal handler to prevent OnLocalCommandAvailable() itself being interrupted
+        by command signals. So it can work off all pushed in commands sequentially.  
+        """
+        oldhandler = signal.signal(signum, signal.SIG_IGN)
+        OnLocalCommandAvailable()
+        signal.signal(signum, oldhandler)
+
+    signal.signal(smscon.smscon_daemon.TELL_COMMAND, commandSignalHandler)
     
     LoadConfiguration() or ExitDaemon('on config load error')
     ValidateIMSI()
@@ -2011,7 +2051,7 @@ def main():
         GpsDevice  = location.GPSDevice()
     
         # set GPS method & most suitable interval
-        gps_interval = int(float(smscnf.GPSINTERVAL))
+        gps_interval = int(smscnf.GPSINTERVAL)
         if   gps_interval <= 10:  gps_interval = location.INTERVAL_10S
         elif gps_interval <= 20:  gps_interval = location.INTERVAL_20S
         elif gps_interval <= 30:  gps_interval = location.INTERVAL_30S
@@ -2024,13 +2064,14 @@ def main():
         GpsDevice.connect('changed', OnGPSchanged, GpsControl )
         GpsControl.connect('gpsd-stopped', OnGPSstop, gobject.MainLoop() )
     
-        if not DoCommandLine():
+        CommandLineStatus = DoCommandLine(sys.argv[1:])
+        if not CommandLineStatus:
             ExitDaemon('bad command line')
 
         InstallSignalReceivers()
 
-        CheckBootLoader(WithNotification = not IsTestModeActive)
-    
+        CheckBootLoader(WithNotification = CommandLineStatus != DoCommandLine.TESTMODE)
+
         gobject.MainLoop().run()
     
     except KeyboardInterrupt:
@@ -2045,7 +2086,4 @@ def main():
     
     ExitDaemon('on mainloop exit')
 
-#############################################
-
-"""
-"""
\ No newline at end of file
+#############################################
\ No newline at end of file
