@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 ##############################################################
 # smscon - remote control library for Nokia N900 phone       #
-# $Id: smsconlib.py 90 2011-12-26 10:35:03Z yablacky $
+# $Id: smsconlib.py 111 2011-12-29 01:18:26Z yablacky $
 ##############################################################
 
 # standard modules
@@ -21,7 +21,7 @@ def getSysBuildNameAndVersion(defaultName, defaultVersion = '999.1', headerInfo
     Analyze information from version control system to find out name and version of a branch.
     @param string defaultName Name to return if version information not recognized.
     @param string defaultVersion Version to return if version information not recognized.
-    @param string headerInfo Value from expanded $Header: https://vcs.maemo.org/svn/smscon/branches/smscon_0.9.8/src/opt/smscon/smsconlib.py 90 2011-12-26 10:35:03Z yablacky $ keyword to be analyzed.
+    @param string headerInfo Value from expanded \$Header\$ keyword to be analyzed.
     @param int dirLevel Number of extra dirs to walk up to find the one that contains version information (0 = parent dir).
     @return list (name, version) The name and version information.
     """
@@ -34,7 +34,7 @@ NAME = 'smscon'
 VERSION = '999.1'   # for trunk and private builds etc.
 
 NAME, VERSION = getSysBuildNameAndVersion(NAME, VERSION,
-    headerInfo  = '$Header: https://vcs.maemo.org/svn/smscon/branches/smscon_0.9.8/src/opt/smscon/smsconlib.py 90 2011-12-26 10:35:03Z yablacky $',
+    headerInfo  = '$Header: https://vcs.maemo.org/svn/smscon/branches/smscon_0.9.9/src/opt/smscon/smsconlib.py 111 2011-12-29 01:18:26Z yablacky $',
     dirLevels   = 3 )
     
 XDEBUG   = False        # cross debug (use only if not running on phone)
@@ -134,7 +134,7 @@ def pidof(ProcessName):
     if PIDs != '':
         return PIDs
     PIDs = []
-    for psLine in os.popen('ps -e -o pid,user,vsz,stat,cmd|grep -v grep|grep %s' % shellArg(' ' + ProcessName)).readlines():
+    for psLine in os.popen('busybox ps|grep -v grep|grep %s' % shellArg(' ' + ProcessName)).readlines():
         PIDs.append(psLine.split()[0])
     return ' '.join(PIDs)
 
@@ -786,9 +786,9 @@ class UserConfiguration(Configuration):
         'LOG_TIMEFORMAT':   ('%',           '%Y-%m-%d %H:%M:%S',    _OPT_PLAIN, '# Format to use for timestamps (see strftime):'),
 
         # GPS settings
-        'GPSTIMEOUT':       (_FloatPattern,     '1800',             _OPT_PLAIN, '# Seconds until give up acquiring GPS coordinates:'),
-        'GPSPOLLING':       (_IntPattern,       '3',                _OPT_PLAIN, '# Number of GSP coordinate acquisitions for COM_LOCATIONs best of:'),
-        'GPSINTERVAL':      (_FloatPattern,     '60',               _OPT_PLAIN, '# Seconds between GSP coordinate acquisitions for COM_TRACKON and COM_LOCATION (10..120:10|20|30|60|120)'),
+        'GPSTIMEOUT':       (_IntPattern,       '1800',             _OPT_PLAIN, '# Seconds until give up acquiring GPS coordinates:'),
+        'GPSPOLLING':       (_IntPattern,       '3',                _OPT_PLAIN, '# Number of GPS coordinate acquisitions for COM_LOCATIONs best of:'),
+        'GPSINTERVAL':      (_IntPattern,       '60',               _OPT_PLAIN, '# Seconds between GPS coordinate acquisitions for COM_TRACKON and COM_LOCATION (10..120:10|20|30|60|120)'),
         'GPSSEND':          ('^sms|email|both$','sms',              _OPT_PLAIN, '# How to send GPS notifications (sms|email|both):'),
 
         # Miscellaneous settings
@@ -821,7 +821,7 @@ class UserConfiguration(Configuration):
     def update(self, ConfigVar, ConfigValue):
         """Same as base class with support for automatic _APPVERSION update."""
         
-        # Prevent to commands being defined as he same string, except the empty string (which disables):
+        # Prevent to commands being defined as the same string, except the empty string (which disables):
         if ConfigVar.startswith(self.SmsCommandPreFix) and ConfigValue != None and ConfigValue != "":
             for OtherVar in self.getSmsCommandList():
                 if OtherVar != ConfigVar and self[OtherVar] == ConfigValue:
@@ -884,6 +884,14 @@ class smscon_daemon:
     TELL_RELOAD    = signal.SIGINT
     TELL_TERMINATE = signal.SIGTERM
     TELL_COMMAND   = signal.SIGHUP
+    _cmdPrefix     = '<cmd>'
+    _cmdSuffix     = '</cmd>'
+
+    def __init__(self):
+        self.commandChannel = None
+
+    def __del__(self):
+        self.closeCommandChannel()
 
     def runs(self):
         """
@@ -897,13 +905,13 @@ class smscon_daemon:
         Start new smscon_daemon instance (even if some daemon instances already run).
         @return void.
         """
-        DaemonArgs = IF(Mode or Function, ' %s "%s" ' % (Mode, Function), '')
+        DaemonArgs = IF(Mode or Function, ' %s %s' % (shellArg(Mode or ''), shellArg(Function or '')), '')
         os.system( '"%s"%s&' % (os.path.join(InstPath, DaemonFile), DaemonArgs))
         LOGGER.info('Daemon %s requested to start%s.' % (DaemonFile, DaemonArgs))
     
     def stop(self):
         """
-        Kill smscon_daemon(s).
+        Kill all smscon_daemon(s).
         @return: string with PIDs of daemons stopped. Empty if none were running.
         """
         PIDs = self.tell(self.TELL_TERMINATE)
@@ -915,14 +923,145 @@ class smscon_daemon:
     
     def tell(self, SignalNumber):
         """
-        Signal smscon_daemon(s).
+        Send a signal to all running smscon_daemon(s).
+        @param int SignalNumber The signal to be sent to the daemons.
         @return: string With list of daemons signaled.
         """
         PIDs = self.runs()
         if PIDs:
             os.system('kill -%d %s' % (abs(SignalNumber), PIDs))
         return PIDs
-    
+
+    def openCommandChannel(self, Mode, Silent = False):
+        if self.commandChannel != None:
+            return True     # TODO: Check if is open in the requested Mode.  
+
+        isReadMode = not 'w' in Mode
+        ModeName = IF(isReadMode, 'reading', 'writing')
+        try:
+            self.commandChannel = os.open(os.path.join(InstPath, CommandFile),
+                                IF(isReadMode, os.O_RDONLY, os.O_WRONLY) | os.O_NONBLOCK)
+            if isReadMode:
+                self.commandChannel = os.fdopen(self.commandChannel)
+        except OSError, e:
+            if e.errno == os.errno.ENXIO: # may happen os.O_WRONLY only
+                if not Silent:
+                    LOGGER.warning('Failed to open instruction channel for %s: Daemon %s was not running.' % (ModeName, DaemonFile))
+                return False
+            if not Silent:
+                LOGGER.error('Failed to open instruction channel for %s: %s' % (ModeName, e))
+            return False
+        except Exception, e:
+            if not Silent:
+                LOGGER.error('Failed to open instruction channel for %s: %s' % (ModeName, e))
+            return False
+        return True
+
+    def closeCommandChannel(self):
+        """
+        Close the command channel if it was open.
+        @return bool Success indicator.
+        """
+        if self.commandChannel == None:
+            return True
+        try:
+            if type(self.commandChannel).__name__ == 'int':
+                os.close(self.commandChannel)
+            else:
+                self.commandChannel.close()
+            self.commandChannel = None
+        except Exception, e:
+            if LOGGER:
+                LOGGER.error('Failed to close instruction channel: %s' % e)
+            self.commandChannel = None  # fail only once.
+            return False
+        return True
+
+    def writeCommand(self, Command = None):
+        """
+        Send an synchroneous command to control a running daemon.
+        @param string|None Command. A None-Command may be used to check if a daemon is
+                    currently listening for commands without actually sending a command. 
+        @return bool True if the command has been send. False on errors or if no daemon is listening.
+        """
+        if not self.openCommandChannel('w'):
+            return False
+        if Command == None:
+            return True;
+        Command = str(Command).strip()
+        if Command != '':
+            while True:
+                try:
+                    os.write(self.commandChannel, self._cmdPrefix + Command + self._cmdSuffix + '\n')
+                except IOError, e:
+                    if e.errno == os.errno.EPIPE:
+                        self.closeCommandChannel()
+                        if not self.openCommandChannel('w'):
+                            return False
+                        continue
+                    LOGGER.error('Failed to write instruction channel: %s' % e)
+                    return False
+                except Exception, e:
+                    LOGGER.error('Failed to write instruction channel: %s' % e)
+                    return False
+                break
+        self.tell(self.TELL_COMMAND)
+        return True 
+
+    def writeCliCommand(self, *args):
+        """
+        Send an synchroneous "command-line-style" command to control a running daemon.
+        @return bool True if the command has been send. False on errors or if no daemon is listening.
+        """
+        if len(args) == 0:
+            return self.writeCommand(None)
+        else:   # TODO: quote xml entities etc...
+            return self.writeCommand("<cli><a>%s</a></cli>" % '</a><a>'.join(args))
+
+    def readCommand(self):
+        """
+        Read an asynchroneous command from a daemon controller.
+        This method must be called repeatedly unless it returns empty string.
+        @return string One command string. Empty if no command.
+        """
+        if not self.openCommandChannel('r'):
+            return ''
+        while True:
+            try:
+                Command = self.commandChannel.readline()
+            except IOError, e:
+                if e.errno == os.errno.EAGAIN:
+                    return ''
+                if e.errno == os.errno.EPIPE:
+                    self.closeCommandChannel()
+                    if not self.openCommandChannel('r'):
+                        return ''
+                    continue
+                LOGGER.error('Failed read instruction channel: %s' % e)
+                return ''
+            except Exception, e:
+                LOGGER.error('Failed read instruction channel: %s' % e)
+                return ''
+            if Command == '':
+                return ''
+            Command = Command.strip()
+            if not Command.startswith(self._cmdPrefix) or not Command.endswith(self._cmdSuffix):
+                LOGGER.warning('Garbage found in command channel: %s' % Command)
+            else:
+                Command = Command[len(self._cmdPrefix):-len(self._cmdSuffix)]
+                if Command != '':   # dont return empty command; caller would thing it was eof
+                    return Command
+
+    def isCliCommand(self, cmd):
+        """
+        Check if string received by readCommand() is a command-line-style command
+        @param string cmd The command received.
+        @return list|False The list of command line args or False if not command line args.  
+        """ 
+        if cmd.startswith('<cli><a>') and cmd.endswith('</a></cli>'):
+            return cmd[8:-10].split('</a><a>')  # TODO: convert quoted xml entities
+        return False
+
 ##############################################################
 # remote shell connection interface 
 
@@ -952,7 +1091,7 @@ class smscon_remote_sh(dict):
         # -- return os.popen('pidof ssh').read().strip('\n') != ''
         PIDs = []
         # Do not use the complete command line for grep because ps may show it only party.
-        for psLine in os.popen('ps|grep -v grep|grep "%s"' % self._getSSHcmdline(2)).readlines():
+        for psLine in os.popen('busybox ps|grep -v grep|grep "%s"' % re.escape(self._getSSHcmdline(2))).readlines():
             PIDs.append(psLine.split()[0])
         return ' '.join(PIDs)
 
@@ -1000,7 +1139,7 @@ class smscon_script:
         @return bool True iff the script file exists or could be created.
         """
         ScriptTemplate = """#!/bin/sh
-# %s custom script (initialized by %s version %s)\n' """ % (NAME, NAME, VERSION) + """
+# %s custom script (initialized by %s version %s) """ % (NAME, NAME, VERSION) + """
 # Script will start on COM_CUSTOM command and executes asynchronously.
 # Script output (stdout and stderr) can be retrieved by COM_CUSTOMLOG command.
 
